@@ -37,10 +37,10 @@ gboolean mapdb_initiate_update(RepoData *repo, gint zoom, gint tilex,
 guint mut_exists_hashfunc(const MapUpdateTask *a);
 gboolean mut_exists_equalfunc(const MapUpdateTask *a, const MapUpdateTask *b);
 gint mut_priority_comparefunc(const MapUpdateTask *a, const MapUpdateTask *b);
-gboolean thread_proc_mut();
+gboolean thread_proc_mut(void);
 
-gboolean repoman_dialog();
+gboolean repoman_dialog(void);
 
-gboolean mapman_dialog();
+gboolean mapman_dialog(void);
 
 #endif /* ifndef MAEMO_MAPPER_MAPS_H */
